@@ -28,9 +28,12 @@
 #include <evince-document.h>
 #include "ev-page-action-widget.h"
 
+#define COMPLETION_RESULTS_WIDTH 50
+
 /* Widget we pass back */
 static void  ev_page_action_widget_init       (EvPageActionWidget      *action_widget);
 static void  ev_page_action_widget_class_init (EvPageActionWidgetClass *action_widget);
+static gboolean ev_page_action_widget_completion_search_is_enabled (EvPageActionWidget *proxy);
 
 enum
 {
@@ -51,6 +54,7 @@ struct _EvPageActionWidget
 	gulong notify_document_signal_id;
 	GtkTreeModel *filter_model;
 	GtkTreeModel *model;
+	GtkEntryCompletion *completion;
 };
 
 static guint widget_signals[WIDGET_N_SIGNALS] = {0, };
@@ -175,6 +179,7 @@ activate_cb (EvPageActionWidget *action_widget)
 	EvLinkAction *link_action;
 	EvLink *link;
 	gchar *link_text;
+	gchar *new_text;
 	gint current_page;
 
 	model = action_widget->doc_model;
@@ -182,6 +187,15 @@ activate_cb (EvPageActionWidget *action_widget)
 
 	text = gtk_entry_get_text (GTK_ENTRY (action_widget->entry));
 
+	/* If we are not in search mode, i.e. we are entering a page number */
+	if (!ev_page_action_widget_completion_search_is_enabled (action_widget)) {
+		/* Convert utf8 fullwidth numbers (eg. japanese) to halfwidth - fixes #1518 */
+		new_text = g_utf8_normalize (text, -1, G_NORMALIZE_ALL);
+		gtk_entry_set_text (GTK_ENTRY (action_widget->entry), new_text);
+		text = gtk_entry_get_text (GTK_ENTRY (action_widget->entry));
+		g_free (new_text);
+	}
+
 	link_dest = ev_link_dest_new_page_label (text);
 	link_action = ev_link_action_new_dest (link_dest);
 	link_text = g_strdup_printf (_("Page %s"), text);
@@ -198,11 +212,22 @@ activate_cb (EvPageActionWidget *action_widget)
 		ev_page_action_widget_set_current_page (action_widget, current_page);
 }
 
+static gboolean
+disable_completion_search (EvPageActionWidget *action_widget)
+{
+	ev_page_action_widget_enable_completion_search (action_widget, FALSE);
+	return G_SOURCE_REMOVE;
+}
+
 static gboolean
 focus_out_cb (EvPageActionWidget *action_widget)
 {
         ev_page_action_widget_set_current_page (action_widget,
                                                 ev_document_model_get_page (action_widget->doc_model));
+        g_object_set (action_widget->entry, "xalign", 1.0, NULL);
+        ev_page_action_widget_update_max_width (action_widget);
+        g_idle_add ((GSourceFunc)disable_completion_search, action_widget);
+
         return FALSE;
 }
 
@@ -339,6 +364,8 @@ ev_page_action_widget_finalize (GObject *object)
 		action_widget->doc_model = NULL;
 	}
 
+	g_clear_object (&action_widget->completion);
+
         ev_page_action_widget_set_document (action_widget, NULL);
 
 	G_OBJECT_CLASS (ev_page_action_widget_parent_class)->finalize (object);
@@ -564,6 +591,8 @@ ev_page_action_widget_update_links_model (EvPageActionWidget *proxy, GtkTreeMode
 	filter_model = get_filter_model_from_model (model);
 
 	completion = gtk_entry_completion_new ();
+	g_clear_object (&proxy->completion);
+	proxy->completion = completion;
 	g_object_set (G_OBJECT (completion),
 		      "popup-set-width", FALSE,
 		      "model", filter_model,
@@ -578,7 +607,7 @@ ev_page_action_widget_update_links_model (EvPageActionWidget *proxy, GtkTreeMode
 	renderer = (GtkCellRenderer *)
 		g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
 			      "ellipsize", PANGO_ELLIPSIZE_END,
-			      "width_chars", 30,
+			      "width_chars", COMPLETION_RESULTS_WIDTH,
 			      NULL);
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion), renderer, TRUE);
 	gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (completion),
@@ -586,8 +615,6 @@ ev_page_action_widget_update_links_model (EvPageActionWidget *proxy, GtkTreeMode
 					    (GtkCellLayoutDataFunc) display_completion_text,
 					    proxy, NULL);
 	gtk_entry_set_completion (GTK_ENTRY (proxy->entry), completion);
-
-	g_object_unref (completion);
 }
 
 void
@@ -596,3 +623,35 @@ ev_page_action_widget_grab_focus (EvPageActionWidget *proxy)
 	gtk_widget_grab_focus (proxy->entry);
 }
 
+void
+ev_page_action_widget_clear (EvPageActionWidget *proxy)
+{
+	gtk_entry_set_text (GTK_ENTRY (proxy->entry), "");
+}
+
+/* Sets width of the text entry, width will be restablished
+ * to default one on focus_out event. This function is used
+ * when searching the Outline, so the user has more space
+ * to write the search term. */
+void
+ev_page_action_widget_set_temporary_entry_width (EvPageActionWidget *proxy, gint width)
+{
+	gtk_entry_set_width_chars (GTK_ENTRY (proxy->entry), width);
+	/* xalign will also be restablished on focus_out */
+	g_object_set (proxy->entry, "xalign", 0., NULL);
+}
+
+/* Enables or disables the completion search on @proxy according to @enable */
+void
+ev_page_action_widget_enable_completion_search (EvPageActionWidget *proxy, gboolean enable)
+{
+	GtkEntryCompletion *completion = enable ? proxy->completion : NULL;
+	gtk_entry_set_completion (GTK_ENTRY (proxy->entry), completion);
+}
+
+/* Returns whether the completion search is enabled in @proxy */
+static gboolean
+ev_page_action_widget_completion_search_is_enabled (EvPageActionWidget *proxy)
+{
+	return gtk_entry_get_completion (GTK_ENTRY (proxy->entry)) != NULL;
+}
\ No newline at end of file
