@@ -2,6 +2,7 @@
  *  this file is part of evince, a gnome document viewer
  *
  * Copyright (C) 2010 Carlos Garcia Campos  <carlosgc@gnome.org>
+ * Copyright (C) 2020 Germán Poo-Caamaño  <gpoo@gnome.org>
  *
  * Evince is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -40,6 +41,13 @@ enum {
         N_COLUMNS
 };
 
+enum {
+        ACTIVATED,
+        N_SIGNALS
+};
+
+static guint signals[N_SIGNALS] = { 0 };
+
 struct _EvSidebarBookmarksPrivate {
         EvDocumentModel *model;
         EvBookmarks     *bookmarks;
@@ -50,13 +58,14 @@ struct _EvSidebarBookmarksPrivate {
 
         /* Popup menu */
         GtkWidget       *popup;
-        GtkUIManager    *ui_manager;
-        GtkActionGroup  *action_group;
 };
 
 static void ev_sidebar_bookmarks_page_iface_init (EvSidebarPageInterface *iface);
 static void ev_sidebar_bookmarks_selection_changed (GtkTreeSelection   *selection,
 						    EvSidebarBookmarks *sidebar_bookmarks);
+static void ev_sidebar_bookmarks_page_changed (EvSidebarBookmarks *sidebar_bookmarks,
+                                               gint                old_page,
+                                               gint                new_page);
 
 G_DEFINE_TYPE_EXTENDED (EvSidebarBookmarks,
                         ev_sidebar_bookmarks,
@@ -66,14 +75,6 @@ G_DEFINE_TYPE_EXTENDED (EvSidebarBookmarks,
                         G_IMPLEMENT_INTERFACE (EV_TYPE_SIDEBAR_PAGE,
                                                ev_sidebar_bookmarks_page_iface_init))
 
-static const gchar popup_menu_ui[] =
-        "<popup name=\"BookmarksPopup\" action=\"BookmarksPopupAction\">\n"
-        "  <menuitem name=\"OpenBookmark\" action=\"OpenBookmark\"/>\n"
-        "  <separator/>\n"
-        "  <menuitem name=\"RenameBookmark\" action=\"RenameBookmark\"/>\n"
-        "  <menuitem name=\"RemoveBookmark\" action=\"RemoveBookmark\"/>\n"
-        "</popup>\n";
-
 static gint
 ev_sidebar_bookmarks_get_selected_page (EvSidebarBookmarks *sidebar_bookmarks,
                                         GtkTreeSelection   *selection)
@@ -94,23 +95,27 @@ ev_sidebar_bookmarks_get_selected_page (EvSidebarBookmarks *sidebar_bookmarks,
 }
 
 static void
-ev_bookmarks_popup_cmd_open_bookmark (GtkAction          *action,
-                                      EvSidebarBookmarks *sidebar_bookmarks)
+ev_bookmarks_popup_cmd_open_bookmark (GSimpleAction *action,
+				      GVariant      *parameter,
+                                      gpointer       sidebar_bookmarks)
 {
-        EvSidebarBookmarksPrivate *priv = sidebar_bookmarks->priv;
+        EvSidebarBookmarksPrivate *priv = EV_SIDEBAR_BOOKMARKS (sidebar_bookmarks)->priv;
         GtkTreeSelection          *selection;
         gint                       page;
+        gint old_page = ev_document_model_get_page (priv->model);
 
         selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->tree_view));
         page = ev_sidebar_bookmarks_get_selected_page (sidebar_bookmarks, selection);
+        g_signal_emit (sidebar_bookmarks, signals[ACTIVATED], 0, old_page, page);
         ev_document_model_set_page (priv->model, page);
 }
 
 static void
-ev_bookmarks_popup_cmd_rename_bookmark (GtkAction          *action,
-                                        EvSidebarBookmarks *sidebar_bookmarks)
+ev_bookmarks_popup_cmd_rename_bookmark (GSimpleAction *action,
+					GVariant      *parameter,
+                                        gpointer       sidebar_bookmarks)
 {
-        EvSidebarBookmarksPrivate *priv = sidebar_bookmarks->priv;
+        EvSidebarBookmarksPrivate *priv = EV_SIDEBAR_BOOKMARKS (sidebar_bookmarks)->priv;
         GtkTreeView               *tree_view = GTK_TREE_VIEW (priv->tree_view);
         GtkTreeSelection          *selection;
         GtkTreeModel              *model;
@@ -130,10 +135,11 @@ ev_bookmarks_popup_cmd_rename_bookmark (GtkAction          *action,
 }
 
 static void
-ev_bookmarks_popup_cmd_remove_bookmark (GtkAction          *action,
-                                        EvSidebarBookmarks *sidebar_bookmarks)
+ev_bookmarks_popup_cmd_delete_bookmark (GSimpleAction *action,
+					GVariant      *parameter,
+                                        gpointer       sidebar_bookmarks)
 {
-        EvSidebarBookmarksPrivate *priv = sidebar_bookmarks->priv;
+        EvSidebarBookmarksPrivate *priv = EV_SIDEBAR_BOOKMARKS (sidebar_bookmarks)->priv;
         GtkTreeSelection          *selection;
         gint                       page;
         EvBookmark                 bm;
@@ -147,14 +153,23 @@ ev_bookmarks_popup_cmd_remove_bookmark (GtkAction          *action,
 		gtk_widget_set_sensitive (priv->del_button, FALSE);
 }
 
-static const GtkActionEntry popup_entries[] = {
-        { "OpenBookmark", NULL, N_("_Open Bookmark"), NULL,
-          NULL, G_CALLBACK (ev_bookmarks_popup_cmd_open_bookmark) },
-        { "RenameBookmark", NULL, N_("_Rename Bookmark"), NULL,
-          NULL, G_CALLBACK (ev_bookmarks_popup_cmd_rename_bookmark) },
-        { "RemoveBookmark", NULL, N_("_Remove Bookmark"), NULL,
-          NULL, G_CALLBACK (ev_bookmarks_popup_cmd_remove_bookmark) }
-};
+static GActionGroup *
+create_action_group (EvSidebarBookmarks *sidebar_bookmarks) {
+	const GActionEntry popup_entries[] = {
+		{ "open-bookmark", ev_bookmarks_popup_cmd_open_bookmark },
+		{ "rename-bookmark", ev_bookmarks_popup_cmd_rename_bookmark },
+		{ "delete-bookmark", ev_bookmarks_popup_cmd_delete_bookmark }
+	};
+	GSimpleActionGroup *group;
+
+	group = g_simple_action_group_new ();
+	g_action_map_add_action_entries (G_ACTION_MAP (group),
+					 popup_entries,
+					 G_N_ELEMENTS (popup_entries),
+					 sidebar_bookmarks);
+
+	return G_ACTION_GROUP (group);
+}
 
 static gint
 compare_bookmarks (EvBookmark *a,
@@ -223,6 +238,8 @@ ev_sidebar_bookmarks_selection_changed (GtkTreeSelection   *selection,
 
         page = ev_sidebar_bookmarks_get_selected_page (sidebar_bookmarks, selection);
         if (page >= 0) {
+                gint old_page = ev_document_model_get_page (priv->model);
+                g_signal_emit (sidebar_bookmarks, signals[ACTIVATED], 0, old_page, page);
                 ev_document_model_set_page (priv->model, page);
                 gtk_widget_set_sensitive (priv->del_button, TRUE);
         } else {
@@ -230,6 +247,21 @@ ev_sidebar_bookmarks_selection_changed (GtkTreeSelection   *selection,
         }
 }
 
+static void ev_sidebar_bookmarks_page_changed (EvSidebarBookmarks *sidebar_bookmarks,
+                                               gint                old_page,
+                                               gint                new_page)
+{
+        EvSidebarBookmarksPrivate *priv = sidebar_bookmarks->priv;
+        GtkTreeSelection          *selection;
+        gint                       selected_page;
+
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->tree_view));
+        selected_page = ev_sidebar_bookmarks_get_selected_page (sidebar_bookmarks, selection);
+
+        if (selected_page != new_page)
+                gtk_tree_selection_unselect_all (selection);
+}
+
 static void
 ev_sidebar_bookmarks_del_clicked (GtkWidget          *button,
                                   EvSidebarBookmarks *sidebar_bookmarks)
@@ -321,68 +353,54 @@ ev_sidebar_bookmarks_query_tooltip (GtkWidget          *widget,
 }
 
 static gboolean
-ev_sidebar_bookmarks_popup_menu_show (EvSidebarBookmarks *sidebar_bookmarks,
-                                      gint                x,
-                                      gint                y,
-                                      gboolean            keyboard_mode)
+ev_sidebar_bookmarks_button_press_cb (GtkWidget          *widget,
+				      GdkEventButton     *event,
+				      EvSidebarBookmarks *sidebar_bookmarks)
 {
-        EvSidebarBookmarksPrivate *priv = sidebar_bookmarks->priv;
-        GtkTreeView               *tree_view = GTK_TREE_VIEW (sidebar_bookmarks->priv->tree_view);
-        GtkTreeSelection          *selection = gtk_tree_view_get_selection (tree_view);
+	EvSidebarBookmarksPrivate *priv = sidebar_bookmarks->priv;
+	GtkTreeView               *tree_view = GTK_TREE_VIEW (sidebar_bookmarks->priv->tree_view);
+	GtkTreeSelection          *selection = gtk_tree_view_get_selection (tree_view);
+	GtkTreePath *path;
 
-        if (keyboard_mode) {
-                if (!gtk_tree_selection_get_selected (selection, NULL, NULL))
-                        return FALSE;
-        } else {
-                GtkTreePath *path;
+	if (!gdk_event_triggers_context_menu ((const GdkEvent *) event))
+		return GDK_EVENT_PROPAGATE;
 
-                if (!gtk_tree_view_get_path_at_pos (tree_view, x, y, &path, NULL, NULL, NULL))
-                        return FALSE;
+	if (!gtk_tree_view_get_path_at_pos (tree_view, event->x, event->y,
+					    &path, NULL, NULL, NULL))
+		return GDK_EVENT_PROPAGATE;
 
-                g_signal_handlers_block_by_func (selection,
-                                                 ev_sidebar_bookmarks_selection_changed,
-                                                 sidebar_bookmarks);
-                gtk_tree_view_set_cursor (tree_view, path, NULL, FALSE);
-                g_signal_handlers_unblock_by_func (selection,
-                                                   ev_sidebar_bookmarks_selection_changed,
-                                                   sidebar_bookmarks);
-		if (!gtk_widget_get_sensitive (priv->del_button))
-			gtk_widget_set_sensitive (priv->del_button, TRUE);
-                gtk_tree_path_free (path);
-        }
+	g_signal_handlers_block_by_func (selection,
+					 ev_sidebar_bookmarks_selection_changed,
+					 sidebar_bookmarks);
+	gtk_tree_view_set_cursor (tree_view, path, NULL, FALSE);
+	g_signal_handlers_unblock_by_func (selection,
+					   ev_sidebar_bookmarks_selection_changed,
+					   sidebar_bookmarks);
+	gtk_tree_path_free (path);
 
-        if (!priv->popup)
-                priv->popup = gtk_ui_manager_get_widget (priv->ui_manager, "/BookmarksPopup");
+	if (!gtk_widget_get_sensitive (priv->del_button))
+		gtk_widget_set_sensitive (priv->del_button, TRUE);
 
-        if (keyboard_mode) {
-                ev_gui_menu_popup_at_tree_view_selection (GTK_MENU (priv->popup),
-                                                          tree_view);
-        } else {
-                gtk_menu_popup_at_pointer (GTK_MENU (priv->popup), NULL);
-        }
+	gtk_menu_popup_at_pointer (GTK_MENU (priv->popup), NULL);
 
-        return TRUE;
-}
-
-static gboolean
-ev_sidebar_bookmarks_button_press (GtkWidget          *widget,
-                                   GdkEventButton     *event,
-                                   EvSidebarBookmarks *sidebar_bookmarks)
-{
-        if (event->button != 3)
-                return FALSE;
-
-        return ev_sidebar_bookmarks_popup_menu_show (sidebar_bookmarks, event->x, event->y, FALSE);
+	return GDK_EVENT_STOP;
 }
 
 static gboolean
 ev_sidebar_bookmarks_popup_menu (GtkWidget *widget)
 {
-        EvSidebarBookmarks *sidebar_bookmarks = EV_SIDEBAR_BOOKMARKS (widget);
-        gint                x, y;
+	EvSidebarBookmarks *sidebar_bookmarks = EV_SIDEBAR_BOOKMARKS (widget);
+	EvSidebarBookmarksPrivate *priv = sidebar_bookmarks->priv;
+	GtkTreeView               *tree_view = GTK_TREE_VIEW (sidebar_bookmarks->priv->tree_view);
+	GtkTreeSelection          *selection = gtk_tree_view_get_selection (tree_view);
 
-        ev_document_misc_get_pointer_position (widget, &x, &y);
-        return ev_sidebar_bookmarks_popup_menu_show (sidebar_bookmarks, x, y, TRUE);
+	if (!gtk_tree_selection_get_selected (selection, NULL, NULL))
+		return FALSE;
+
+	ev_gui_menu_popup_at_tree_view_selection (GTK_MENU (priv->popup),
+						  tree_view);
+
+	return TRUE;
 }
 
 static void
@@ -401,16 +419,6 @@ ev_sidebar_bookmarks_dispose (GObject *object)
                 priv->bookmarks = NULL;
         }
 
-        if (priv->action_group) {
-                g_object_unref (priv->action_group);
-                priv->action_group = NULL;
-        }
-
-        if (priv->ui_manager) {
-                g_object_unref (priv->ui_manager);
-                priv->ui_manager = NULL;
-        }
-
         G_OBJECT_CLASS (ev_sidebar_bookmarks_parent_class)->dispose (object);
 }
 
@@ -418,89 +426,22 @@ static void
 ev_sidebar_bookmarks_init (EvSidebarBookmarks *sidebar_bookmarks)
 {
         EvSidebarBookmarksPrivate *priv;
-        GtkWidget                 *swindow;
-        GtkWidget                 *hbox;
-        GtkListStore              *model;
-        GtkCellRenderer           *renderer;
-        GtkTreeSelection          *selection;
+	GtkBuilder                *builder;
+	GMenuModel                *popup_model;
 
-        sidebar_bookmarks->priv = ev_sidebar_bookmarks_get_instance_private (sidebar_bookmarks);
-        priv = sidebar_bookmarks->priv;
+	sidebar_bookmarks->priv = ev_sidebar_bookmarks_get_instance_private (sidebar_bookmarks);
+	priv = sidebar_bookmarks->priv;
 
-        swindow = gtk_scrolled_window_new (NULL, NULL);
-        gtk_box_pack_start (GTK_BOX (sidebar_bookmarks), swindow, TRUE, TRUE, 0);
-        gtk_widget_show (swindow);
-
-        model = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_UINT);
-        priv->tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
-        g_object_unref (model);
-        g_signal_connect (priv->tree_view, "query-tooltip",
-                          G_CALLBACK (ev_sidebar_bookmarks_query_tooltip),
-                          sidebar_bookmarks);
-        g_signal_connect (priv->tree_view,
-                          "button-press-event",
-                          G_CALLBACK (ev_sidebar_bookmarks_button_press),
-                          sidebar_bookmarks);
-        gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (priv->tree_view), FALSE);
-        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->tree_view));
-        g_signal_connect (selection, "changed",
-                          G_CALLBACK (ev_sidebar_bookmarks_selection_changed),
-                          sidebar_bookmarks);
-
-        renderer = gtk_cell_renderer_text_new ();
-        g_object_set (renderer,
-                      "ellipsize", PANGO_ELLIPSIZE_END,
-                      "editable", TRUE,
-                      NULL);
-        g_signal_connect (renderer, "edited",
-                          G_CALLBACK (ev_sidebar_bookmarks_bookmark_renamed),
-                          sidebar_bookmarks);
-        gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (priv->tree_view),
-                                                     0, NULL, renderer,
-                                                     "markup", COLUMN_MARKUP,
-                                                     NULL);
-        gtk_container_add (GTK_CONTAINER (swindow), priv->tree_view);
-        gtk_widget_show (priv->tree_view);
-
-        hbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
-        g_object_set (hbox, "margin", 6, NULL);
-        gtk_widget_set_halign (hbox, GTK_ALIGN_CENTER);
-        gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_EXPAND);
-	gtk_box_set_homogeneous (GTK_BOX (hbox), FALSE);
-
-        priv->add_button = gtk_button_new_from_icon_name ("list-add-symbolic", GTK_ICON_SIZE_MENU);
-        gtk_widget_set_tooltip_text (priv->add_button, _("Add bookmark"));
-        atk_object_set_name (gtk_widget_get_accessible (priv->add_button), _("Add bookmark"));
-        gtk_actionable_set_action_name (GTK_ACTIONABLE (priv->add_button),
-                                        "win.add-bookmark");
-        gtk_widget_set_sensitive (priv->add_button, FALSE);
-        gtk_box_pack_start (GTK_BOX (hbox), priv->add_button, FALSE, FALSE, 0);
-        gtk_widget_show (priv->add_button);
-
-        priv->del_button = gtk_button_new_from_icon_name ("list-remove-symbolic", GTK_ICON_SIZE_MENU);
-        gtk_widget_set_tooltip_text (priv->del_button, _("Remove bookmark"));
-        atk_object_set_name (gtk_widget_get_accessible (priv->del_button), _("Remove bookmark"));
-        g_signal_connect (priv->del_button, "clicked",
-                          G_CALLBACK (ev_sidebar_bookmarks_del_clicked),
-                          sidebar_bookmarks);
-        gtk_widget_set_sensitive (priv->del_button, FALSE);
-        gtk_box_pack_start (GTK_BOX (hbox), priv->del_button, FALSE, FALSE, 0);
-        gtk_widget_show (priv->del_button);
-
-        gtk_box_pack_end (GTK_BOX (sidebar_bookmarks), hbox, FALSE, TRUE, 0);
-        gtk_widget_show (hbox);
-        gtk_widget_show (GTK_WIDGET (sidebar_bookmarks));
+	gtk_widget_init_template (GTK_WIDGET (sidebar_bookmarks));
 
         /* Popup menu */
-        priv->action_group = gtk_action_group_new ("BookmarsPopupActions");
-        gtk_action_group_set_translation_domain (priv->action_group, NULL);
-        gtk_action_group_add_actions (priv->action_group, popup_entries,
-                                      G_N_ELEMENTS (popup_entries),
-                                      sidebar_bookmarks);
-        priv->ui_manager = gtk_ui_manager_new ();
-        gtk_ui_manager_insert_action_group (priv->ui_manager,
-                                            priv->action_group, 0);
-        gtk_ui_manager_add_ui_from_string (priv->ui_manager, popup_menu_ui, -1, NULL);
+	builder = gtk_builder_new_from_resource ("/org/gnome/evince/gtk/menus.ui");
+	popup_model = g_object_ref (G_MENU_MODEL (gtk_builder_get_object (builder, "bookmarks-popup")));
+	priv->popup = gtk_menu_new_from_model (popup_model);
+	gtk_widget_insert_action_group (priv->popup,
+					"bookmarks",
+					create_action_group (sidebar_bookmarks));
+	g_object_unref (builder);
 }
 
 static void
@@ -533,16 +474,36 @@ ev_sidebar_bookmarks_class_init (EvSidebarBookmarksClass *klass)
         g_object_class->dispose = ev_sidebar_bookmarks_dispose;
 
         widget_class->popup_menu = ev_sidebar_bookmarks_popup_menu;
+	gtk_widget_class_set_template_from_resource (widget_class,
+			"/org/gnome/evince/ui/sidebar-bookmarks.ui");
+	gtk_widget_class_bind_template_child_private (widget_class, EvSidebarBookmarks, tree_view);
+	gtk_widget_class_bind_template_child_private (widget_class, EvSidebarBookmarks, del_button);
+	gtk_widget_class_bind_template_child_private (widget_class, EvSidebarBookmarks, add_button);
+
+	gtk_widget_class_bind_template_callback (widget_class, ev_sidebar_bookmarks_del_clicked);
+	gtk_widget_class_bind_template_callback (widget_class, ev_sidebar_bookmarks_bookmark_renamed);
+	gtk_widget_class_bind_template_callback (widget_class, ev_sidebar_bookmarks_query_tooltip);
+	gtk_widget_class_bind_template_callback (widget_class, ev_sidebar_bookmarks_selection_changed);
+	gtk_widget_class_bind_template_callback (widget_class, ev_sidebar_bookmarks_query_tooltip);
+	gtk_widget_class_bind_template_callback (widget_class, ev_sidebar_bookmarks_button_press_cb);
 
         g_object_class_override_property (g_object_class, PROP_WIDGET, "main-widget");
+	/* Signals */
+        signals[ACTIVATED] =
+                g_signal_new ("bookmark-activated",
+                              EV_TYPE_SIDEBAR_BOOKMARKS,
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (EvSidebarBookmarksClass, activated),
+                              NULL, NULL,
+                              NULL,
+                              G_TYPE_NONE, 2,
+                              G_TYPE_INT, G_TYPE_INT);
 }
 
 GtkWidget *
 ev_sidebar_bookmarks_new (void)
 {
-        return GTK_WIDGET (g_object_new (EV_TYPE_SIDEBAR_BOOKMARKS,
-                                         "orientation", GTK_ORIENTATION_VERTICAL,
-                                         NULL));
+        return GTK_WIDGET (g_object_new (EV_TYPE_SIDEBAR_BOOKMARKS, NULL));
 }
 
 void
@@ -581,6 +542,9 @@ ev_sidebar_bookmarks_set_model (EvSidebarPage   *sidebar_page,
         if (priv->model)
                 g_object_unref (priv->model);
         priv->model = g_object_ref (model);
+        g_signal_connect_swapped (model, "page-changed",
+                                  G_CALLBACK (ev_sidebar_bookmarks_page_changed),
+                                  sidebar_page);
 }
 
 static gboolean
